package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TPlatformHourCouponStore;
import com.haomostudio.so.service.TPlatformHourCouponStoreService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TPlatformHourCouponStore")
@Controller
public class TPlatformHourCouponStoreController {

    @Autowired
    TPlatformHourCouponStoreService tPlatformHourCouponStoreService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TPlatformHourCouponStore'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_hour_coupon_stores/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTPlatformHourCouponStore(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "platformHourCouponId", required = false) Integer platformHourCouponId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "storeId", required = false) Integer storeId
    ) {
        TPlatformHourCouponStore item = new TPlatformHourCouponStore();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( platformHourCouponId != null ){
            item.setPlatformHourCouponId(platformHourCouponId);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( storeId != null ){
            item.setStoreId(storeId);
        }
        tPlatformHourCouponStoreService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tPlatformHourCouponStoreService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TPlatformHourCouponStore'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_platform_hour_coupon_stores/{t_platform_hour_coupon_store_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTPlatformHourCouponStore(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_hour_coupon_store_id") Integer id
    ) {
        TPlatformHourCouponStore item = tPlatformHourCouponStoreService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tPlatformHourCouponStoreService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TPlatformHourCouponStore'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_platform_hour_coupon_stores/{t_platform_hour_coupon_store_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTPlatformHourCouponStore(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_hour_coupon_store_id") Integer id,
            @RequestParam(value = "platformHourCouponId", required = false) Integer platformHourCouponId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "storeId", required = false) Integer storeId
    ) {
        TPlatformHourCouponStore item = tPlatformHourCouponStoreService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( platformHourCouponId != null ){
            item.setPlatformHourCouponId(platformHourCouponId);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( storeId != null ){
            item.setStoreId(storeId);
        }
        tPlatformHourCouponStoreService.update(item);

        return tPlatformHourCouponStoreService.get(id);
    }

    @ApiOperation(value = "查询'TPlatformHourCouponStore'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_platform_hour_coupon_stores/{t_platform_hour_coupon_store_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformHourCouponStore(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_hour_coupon_store_id") Integer id,
            @RequestParam(value = "platformHourCouponId", required = false) Integer platformHourCouponId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "storeId", required = false) Integer storeId
    ) {
        if(request.getMethod().equals("GET")){
            TPlatformHourCouponStore item = tPlatformHourCouponStoreService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TPlatformHourCouponStore item = tPlatformHourCouponStoreService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( platformHourCouponId != null ){
                item.setPlatformHourCouponId(platformHourCouponId);
            }
            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            if( storeId != null ){
                item.setStoreId(storeId);
            }
            tPlatformHourCouponStoreService.update(item);

            return tPlatformHourCouponStoreService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TPlatformHourCouponStore item = tPlatformHourCouponStoreService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tPlatformHourCouponStoreService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TPlatformHourCouponStore'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_platform_hour_coupon_stores",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformHourCouponStores(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tPlatformHourCouponStoreService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tPlatformHourCouponStoreService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_platform_hour_coupon_stores/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformHourCouponStoresId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TPlatformHourCouponStore> list = tPlatformHourCouponStoreService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TPlatformHourCouponStore.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TPlatformHourCouponStore'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_hour_coupon_stores/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTPlatformHourCouponStores(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TPlatformHourCouponStore> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TPlatformHourCouponStore item = new TPlatformHourCouponStore();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("platformHourCouponId")   ){
            item.setPlatformHourCouponId(Integer.valueOf(jsonObject.get("platformHourCouponId").toString()));
        }
        if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
        if(jsonObject.containsKey("storeId")   ){
            item.setStoreId(Integer.valueOf(jsonObject.get("storeId").toString()));
        }
             tPlatformHourCouponStoreService.create(item);
             lists.add(tPlatformHourCouponStoreService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TPlatformHourCouponStore'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_platform_hour_coupon_stores/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTPlatformHourCouponStore(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TPlatformHourCouponStore item = tPlatformHourCouponStoreService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tPlatformHourCouponStoreService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
