package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TPlatformServiceProvider;
import com.haomostudio.so.service.TPlatformServiceProviderService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TPlatformServiceProvider")
@Controller
public class TPlatformServiceProviderController {

    @Autowired
    TPlatformServiceProviderService tPlatformServiceProviderService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TPlatformServiceProvider'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_service_providers/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTPlatformServiceProvider(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "providerId", required = false) Integer providerId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "fullName", required = false) String fullName,
            @RequestParam(value = "abbrName", required = false) String abbrName,
            @RequestParam(value = "contact", required = false) String contact,
            @RequestParam(value = "phone", required = false) String phone,
            @RequestParam(value = "introduction", required = false) String introduction,
            @RequestParam(value = "statusCode", required = false) Integer statusCode,
            @RequestParam(value = "isDelete", required = false) Integer isDelete,
            @RequestParam(value = "createDate", required = false) String createDate,
            @RequestParam(value = "modified", required = false) String modified
    ) {
        TPlatformServiceProvider item = new TPlatformServiceProvider();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( providerId != null ){
            item.setProviderId(providerId);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( fullName != null ){
            item.setFullName(fullName);
        }
        if( abbrName != null ){
            item.setAbbrName(abbrName);
        }
        if( contact != null ){
            item.setContact(contact);
        }
        if( phone != null ){
            item.setPhone(phone);
        }
        if( introduction != null ){
            item.setIntroduction(introduction);
        }
        if( statusCode != null ){
            item.setStatusCode(statusCode);
        }
        if( isDelete != null ){
            item.setIsDelete(isDelete);
        }
        if( createDate != null ){
            item.setCreateDate(Tools.convertStringToDate(createDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( modified != null ){
            item.setModified(Tools.convertStringToDate(modified, "yyyy-MM-dd HH:mm:ss"));
        }
        tPlatformServiceProviderService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tPlatformServiceProviderService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TPlatformServiceProvider'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_platform_service_providers/{t_platform_service_provider_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTPlatformServiceProvider(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_service_provider_id") Integer id
    ) {
        TPlatformServiceProvider item = tPlatformServiceProviderService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tPlatformServiceProviderService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TPlatformServiceProvider'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_platform_service_providers/{t_platform_service_provider_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTPlatformServiceProvider(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_service_provider_id") Integer id,
            @RequestParam(value = "providerId", required = false) Integer providerId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "fullName", required = false) String fullName,
            @RequestParam(value = "abbrName", required = false) String abbrName,
            @RequestParam(value = "contact", required = false) String contact,
            @RequestParam(value = "phone", required = false) String phone,
            @RequestParam(value = "introduction", required = false) String introduction,
            @RequestParam(value = "statusCode", required = false) Integer statusCode,
            @RequestParam(value = "isDelete", required = false) Integer isDelete,
            @RequestParam(value = "createDate", required = false) String createDate,
            @RequestParam(value = "modified", required = false) String modified
    ) {
        TPlatformServiceProvider item = tPlatformServiceProviderService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( providerId != null ){
            item.setProviderId(providerId);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        if( fullName != null ){
            item.setFullName(fullName);
        }
        if( abbrName != null ){
            item.setAbbrName(abbrName);
        }
        if( contact != null ){
            item.setContact(contact);
        }
        if( phone != null ){
            item.setPhone(phone);
        }
        if( introduction != null ){
            item.setIntroduction(introduction);
        }
        if( statusCode != null ){
            item.setStatusCode(statusCode);
        }
        if( isDelete != null ){
            item.setIsDelete(isDelete);
        }
        if( createDate != null ){
            item.setCreateDate(Tools.convertStringToDate(createDate, "yyyy-MM-dd HH:mm:ss"));
        }
        if( modified != null ){
            item.setModified(Tools.convertStringToDate(modified, "yyyy-MM-dd HH:mm:ss"));
        }
        tPlatformServiceProviderService.update(item);

        return tPlatformServiceProviderService.get(id);
    }

    @ApiOperation(value = "查询'TPlatformServiceProvider'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_platform_service_providers/{t_platform_service_provider_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformServiceProvider(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_platform_service_provider_id") Integer id,
            @RequestParam(value = "providerId", required = false) Integer providerId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId,
            @RequestParam(value = "fullName", required = false) String fullName,
            @RequestParam(value = "abbrName", required = false) String abbrName,
            @RequestParam(value = "contact", required = false) String contact,
            @RequestParam(value = "phone", required = false) String phone,
            @RequestParam(value = "introduction", required = false) String introduction,
            @RequestParam(value = "statusCode", required = false) Integer statusCode,
            @RequestParam(value = "isDelete", required = false) Integer isDelete,
            @RequestParam(value = "createDate", required = false) String createDate,
            @RequestParam(value = "modified", required = false) String modified
    ) {
        if(request.getMethod().equals("GET")){
            TPlatformServiceProvider item = tPlatformServiceProviderService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TPlatformServiceProvider item = tPlatformServiceProviderService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( providerId != null ){
                item.setProviderId(providerId);
            }
            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            if( fullName != null ){
                item.setFullName(fullName);
            }
            if( abbrName != null ){
                item.setAbbrName(abbrName);
            }
            if( contact != null ){
                item.setContact(contact);
            }
            if( phone != null ){
                item.setPhone(phone);
            }
            if( introduction != null ){
                item.setIntroduction(introduction);
            }
            if( statusCode != null ){
                item.setStatusCode(statusCode);
            }
            if( isDelete != null ){
                item.setIsDelete(isDelete);
            }
            if( createDate != null ){
                item.setCreateDate(Tools.convertStringToDate(createDate, "yyyy-MM-dd HH:mm:ss"));
            }
            if( modified != null ){
                item.setModified(Tools.convertStringToDate(modified, "yyyy-MM-dd HH:mm:ss"));
            }
            tPlatformServiceProviderService.update(item);

            return tPlatformServiceProviderService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TPlatformServiceProvider item = tPlatformServiceProviderService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tPlatformServiceProviderService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TPlatformServiceProvider'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_platform_service_providers",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformServiceProviders(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tPlatformServiceProviderService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tPlatformServiceProviderService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_platform_service_providers/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTPlatformServiceProvidersId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TPlatformServiceProvider> list = tPlatformServiceProviderService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TPlatformServiceProvider.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TPlatformServiceProvider'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_platform_service_providers/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTPlatformServiceProviders(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TPlatformServiceProvider> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TPlatformServiceProvider item = new TPlatformServiceProvider();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("providerId")   ){
            item.setProviderId(Integer.valueOf(jsonObject.get("providerId").toString()));
        }
        if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
        if(jsonObject.containsKey("fullName")   ){
            item.setFullName(jsonObject.get("fullName").toString());
        }
        if(jsonObject.containsKey("abbrName")   ){
            item.setAbbrName(jsonObject.get("abbrName").toString());
        }
        if(jsonObject.containsKey("contact")   ){
            item.setContact(jsonObject.get("contact").toString());
        }
        if(jsonObject.containsKey("phone")   ){
            item.setPhone(jsonObject.get("phone").toString());
        }
        if(jsonObject.containsKey("introduction")   ){
            item.setIntroduction(jsonObject.get("introduction").toString());
        }
        if(jsonObject.containsKey("statusCode")   ){
            item.setStatusCode(Integer.valueOf(jsonObject.get("statusCode").toString()));
        }
        if(jsonObject.containsKey("isDelete")   ){
            item.setIsDelete(Integer.valueOf(jsonObject.get("isDelete").toString()));
        }
        if(jsonObject.containsKey("createDate")   ){
            item.setCreateDate(Tools.convertStringToDate(jsonObject.get("createDate").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("modified")   ){
            item.setModified(Tools.convertStringToDate(jsonObject.get("modified").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
             tPlatformServiceProviderService.create(item);
             lists.add(tPlatformServiceProviderService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TPlatformServiceProvider'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_platform_service_providers/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTPlatformServiceProvider(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TPlatformServiceProvider item = tPlatformServiceProviderService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tPlatformServiceProviderService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
