package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TRole;
import com.haomostudio.so.service.TRoleService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TRole")
@Controller
public class TRoleController {

    @Autowired
    TRoleService tRoleService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TRole'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_roles/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTRole(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "roleName", required = false) String roleName,
            @RequestParam(value = "roleDesc", required = false) String roleDesc,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "createdAdminUserId", required = false) Integer createdAdminUserId,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "updatedAdminUserId", required = false) Integer updatedAdminUserId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId
    ) {
        TRole item = new TRole();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( roleName != null ){
            item.setRoleName(roleName);
        }
        if( roleDesc != null ){
            item.setRoleDesc(roleDesc);
        }
        if( status != null ){
            item.setStatus(status);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( createdAdminUserId != null ){
            item.setCreatedAdminUserId(createdAdminUserId);
        }
        if( updated != null ){
            item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
        }
        if( updatedAdminUserId != null ){
            item.setUpdatedAdminUserId(updatedAdminUserId);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        tRoleService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tRoleService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TRole'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_roles/{t_role_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTRole(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_role_id") Integer id
    ) {
        TRole item = tRoleService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tRoleService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TRole'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_roles/{t_role_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTRole(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_role_id") Integer id,
            @RequestParam(value = "roleName", required = false) String roleName,
            @RequestParam(value = "roleDesc", required = false) String roleDesc,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "createdAdminUserId", required = false) Integer createdAdminUserId,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "updatedAdminUserId", required = false) Integer updatedAdminUserId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId
    ) {
        TRole item = tRoleService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( roleName != null ){
            item.setRoleName(roleName);
        }
        if( roleDesc != null ){
            item.setRoleDesc(roleDesc);
        }
        if( status != null ){
            item.setStatus(status);
        }
        if( created != null ){
            item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
        }
        if( createdAdminUserId != null ){
            item.setCreatedAdminUserId(createdAdminUserId);
        }
        if( updated != null ){
            item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
        }
        if( updatedAdminUserId != null ){
            item.setUpdatedAdminUserId(updatedAdminUserId);
        }
        if( spaceId != null ){
            item.setSpaceId(spaceId);
        }
        tRoleService.update(item);

        return tRoleService.get(id);
    }

    @ApiOperation(value = "查询'TRole'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_roles/{t_role_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTRole(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_role_id") Integer id,
            @RequestParam(value = "roleName", required = false) String roleName,
            @RequestParam(value = "roleDesc", required = false) String roleDesc,
            @RequestParam(value = "status", required = false) Integer status,
            @RequestParam(value = "created", required = false) String created,
            @RequestParam(value = "createdAdminUserId", required = false) Integer createdAdminUserId,
            @RequestParam(value = "updated", required = false) String updated,
            @RequestParam(value = "updatedAdminUserId", required = false) Integer updatedAdminUserId,
            @RequestParam(value = "spaceId", required = false) Integer spaceId
    ) {
        if(request.getMethod().equals("GET")){
            TRole item = tRoleService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TRole item = tRoleService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( roleName != null ){
                item.setRoleName(roleName);
            }
            if( roleDesc != null ){
                item.setRoleDesc(roleDesc);
            }
            if( status != null ){
                item.setStatus(status);
            }
            if( created != null ){
                item.setCreated(Tools.convertStringToDate(created, "yyyy-MM-dd HH:mm:ss"));
            }
            if( createdAdminUserId != null ){
                item.setCreatedAdminUserId(createdAdminUserId);
            }
            if( updated != null ){
                item.setUpdated(Tools.convertStringToDate(updated, "yyyy-MM-dd HH:mm:ss"));
            }
            if( updatedAdminUserId != null ){
                item.setUpdatedAdminUserId(updatedAdminUserId);
            }
            if( spaceId != null ){
                item.setSpaceId(spaceId);
            }
            tRoleService.update(item);

            return tRoleService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TRole item = tRoleService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tRoleService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TRole'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_roles",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTRoles(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tRoleService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tRoleService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_roles/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTRolesId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TRole> list = tRoleService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TRole.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TRole'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_roles/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTRoles(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TRole> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TRole item = new TRole();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("roleName")   ){
            item.setRoleName(jsonObject.get("roleName").toString());
        }
        if(jsonObject.containsKey("roleDesc")   ){
            item.setRoleDesc(jsonObject.get("roleDesc").toString());
        }
        if(jsonObject.containsKey("status")   ){
            item.setStatus(Integer.valueOf(jsonObject.get("status").toString()));
        }
        if(jsonObject.containsKey("created")   ){
            item.setCreated(Tools.convertStringToDate(jsonObject.get("created").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("createdAdminUserId")   ){
            item.setCreatedAdminUserId(Integer.valueOf(jsonObject.get("createdAdminUserId").toString()));
        }
        if(jsonObject.containsKey("updated")   ){
            item.setUpdated(Tools.convertStringToDate(jsonObject.get("updated").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("updatedAdminUserId")   ){
            item.setUpdatedAdminUserId(Integer.valueOf(jsonObject.get("updatedAdminUserId").toString()));
        }
        if(jsonObject.containsKey("spaceId")   ){
            item.setSpaceId(Integer.valueOf(jsonObject.get("spaceId").toString()));
        }
             tRoleService.create(item);
             lists.add(tRoleService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TRole'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_roles/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTRole(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TRole item = tRoleService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tRoleService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
