package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TUnlockRecord;
import com.haomostudio.so.service.TUnlockRecordService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TUnlockRecord")
@Controller
public class TUnlockRecordController {

    @Autowired
    TUnlockRecordService tUnlockRecordService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TUnlockRecord'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_unlock_records/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTUnlockRecord(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "userId", required = false) Integer userId,
            @RequestParam(value = "staffCode", required = false) Integer staffCode,
            @RequestParam(value = "lockBaseId", required = false) Integer lockBaseId,
            @RequestParam(value = "operateTime", required = false) String operateTime,
            @RequestParam(value = "operateType", required = false) Integer operateType,
            @RequestParam(value = "remark", required = false) String remark
    ) {
        TUnlockRecord item = new TUnlockRecord();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( userId != null ){
            item.setUserId(userId);
        }
        if( staffCode != null ){
            item.setStaffCode(staffCode);
        }
        if( lockBaseId != null ){
            item.setLockBaseId(lockBaseId);
        }
        if( operateTime != null ){
            item.setOperateTime(Tools.convertStringToDate(operateTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( operateType != null ){
            item.setOperateType(operateType);
        }
        if( remark != null ){
            item.setRemark(remark);
        }
        tUnlockRecordService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tUnlockRecordService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TUnlockRecord'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_unlock_records/{t_unlock_record_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTUnlockRecord(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_unlock_record_id") Integer id
    ) {
        TUnlockRecord item = tUnlockRecordService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tUnlockRecordService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TUnlockRecord'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_unlock_records/{t_unlock_record_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTUnlockRecord(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_unlock_record_id") Integer id,
            @RequestParam(value = "userId", required = false) Integer userId,
            @RequestParam(value = "staffCode", required = false) Integer staffCode,
            @RequestParam(value = "lockBaseId", required = false) Integer lockBaseId,
            @RequestParam(value = "operateTime", required = false) String operateTime,
            @RequestParam(value = "operateType", required = false) Integer operateType,
            @RequestParam(value = "remark", required = false) String remark
    ) {
        TUnlockRecord item = tUnlockRecordService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( userId != null ){
            item.setUserId(userId);
        }
        if( staffCode != null ){
            item.setStaffCode(staffCode);
        }
        if( lockBaseId != null ){
            item.setLockBaseId(lockBaseId);
        }
        if( operateTime != null ){
            item.setOperateTime(Tools.convertStringToDate(operateTime, "yyyy-MM-dd HH:mm:ss"));
        }
        if( operateType != null ){
            item.setOperateType(operateType);
        }
        if( remark != null ){
            item.setRemark(remark);
        }
        tUnlockRecordService.update(item);

        return tUnlockRecordService.get(id);
    }

    @ApiOperation(value = "查询'TUnlockRecord'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_unlock_records/{t_unlock_record_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTUnlockRecord(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_unlock_record_id") Integer id,
            @RequestParam(value = "userId", required = false) Integer userId,
            @RequestParam(value = "staffCode", required = false) Integer staffCode,
            @RequestParam(value = "lockBaseId", required = false) Integer lockBaseId,
            @RequestParam(value = "operateTime", required = false) String operateTime,
            @RequestParam(value = "operateType", required = false) Integer operateType,
            @RequestParam(value = "remark", required = false) String remark
    ) {
        if(request.getMethod().equals("GET")){
            TUnlockRecord item = tUnlockRecordService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TUnlockRecord item = tUnlockRecordService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( userId != null ){
                item.setUserId(userId);
            }
            if( staffCode != null ){
                item.setStaffCode(staffCode);
            }
            if( lockBaseId != null ){
                item.setLockBaseId(lockBaseId);
            }
            if( operateTime != null ){
                item.setOperateTime(Tools.convertStringToDate(operateTime, "yyyy-MM-dd HH:mm:ss"));
            }
            if( operateType != null ){
                item.setOperateType(operateType);
            }
            if( remark != null ){
                item.setRemark(remark);
            }
            tUnlockRecordService.update(item);

            return tUnlockRecordService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TUnlockRecord item = tUnlockRecordService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tUnlockRecordService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TUnlockRecord'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_unlock_records",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTUnlockRecords(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tUnlockRecordService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tUnlockRecordService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_unlock_records/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTUnlockRecordsId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TUnlockRecord> list = tUnlockRecordService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TUnlockRecord.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TUnlockRecord'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_unlock_records/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTUnlockRecords(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TUnlockRecord> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TUnlockRecord item = new TUnlockRecord();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("userId")   ){
            item.setUserId(Integer.valueOf(jsonObject.get("userId").toString()));
        }
        if(jsonObject.containsKey("staffCode")   ){
            item.setStaffCode(Integer.valueOf(jsonObject.get("staffCode").toString()));
        }
        if(jsonObject.containsKey("lockBaseId")   ){
            item.setLockBaseId(Integer.valueOf(jsonObject.get("lockBaseId").toString()));
        }
        if(jsonObject.containsKey("operateTime")   ){
            item.setOperateTime(Tools.convertStringToDate(jsonObject.get("operateTime").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("operateType")   ){
            item.setOperateType(Integer.valueOf(jsonObject.get("operateType").toString()));
        }
        if(jsonObject.containsKey("remark")   ){
            item.setRemark(jsonObject.get("remark").toString());
        }
             tUnlockRecordService.create(item);
             lists.add(tUnlockRecordService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TUnlockRecord'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_unlock_records/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTUnlockRecord(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TUnlockRecord item = tUnlockRecordService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tUnlockRecordService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
