package com.haomostudio.so.controller;

import com.haomostudio.so.common.Resp;
import com.haomostudio.so.po.TValueAddServiceType;
import com.haomostudio.so.service.TValueAddServiceTypeService;
import com.haomostudio.so.service.HmUtils.Tools;
import com.alibaba.fastjson.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.InvocationTargetException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by hxg on 2016/10/06.
 */
@Api(description = "TValueAddServiceType")
@Controller
public class TValueAddServiceTypeController {

    @Autowired
    TValueAddServiceTypeService tValueAddServiceTypeService;

    @Autowired
    HttpServletResponse response;

    @ApiOperation(value = "创建'TValueAddServiceType'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_value_add_service_types/new",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createTValueAddServiceType(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "type", required = false) Integer type,
            @RequestParam(value = "icon", required = false) String icon,
            @RequestParam(value = "iconsel", required = false) String iconsel,
            @RequestParam(value = "titlePic", required = false) String titlePic,
            @RequestParam(value = "create", required = false) String create,
            @RequestParam(value = "modified", required = false) String modified
    ) {
        TValueAddServiceType item = new TValueAddServiceType();
        String id = Tools.getUUID();
        item.setId(Integer.valueOf(id));
        if( name != null ){
            item.setName(name);
        }
        if( type != null ){
            item.setType(type);
        }
        if( icon != null ){
            item.setIcon(icon);
        }
        if( iconsel != null ){
            item.setIconsel(iconsel);
        }
        if( titlePic != null ){
            item.setTitlePic(titlePic);
        }
        if( create != null ){
            item.setCreate(Tools.convertStringToDate(create, "yyyy-MM-dd HH:mm:ss"));
        }
        if( modified != null ){
            item.setModified(Tools.convertStringToDate(modified, "yyyy-MM-dd HH:mm:ss"));
        }
        tValueAddServiceTypeService.create(item);

        //若request param为null，但字段定义有默认值，则需从数据库中返回结果
        return tValueAddServiceTypeService.get(Integer.valueOf(id));
        //return item;
    }

    @ApiOperation(value = "删除'TValueAddServiceType'表中的某条记录", notes = "根据url传入的数据id，删除整条记录。")
    @RequestMapping(value = "/t_value_add_service_types/{t_value_add_service_type_id}/delete",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object deleteTValueAddServiceType(
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_value_add_service_type_id") Integer id
    ) {
        TValueAddServiceType item = tValueAddServiceTypeService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be deleted doesn't exist");
        }

        tValueAddServiceTypeService.delete(id);
        return Resp.succ("delete success");
    }

    @ApiOperation(value = "修改'TValueAddServiceType'表中的某条记录", notes = "根据url传入的数据id，确定修改表中的某条记录，传入表中字段要修改的信息，不传代表不修改。并返回给View层")
    @RequestMapping(value = "/t_value_add_service_types/{t_value_add_service_type_id}/edit",
                    method = RequestMethod.POST,
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object editTValueAddServiceType(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_value_add_service_type_id") Integer id,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "type", required = false) Integer type,
            @RequestParam(value = "icon", required = false) String icon,
            @RequestParam(value = "iconsel", required = false) String iconsel,
            @RequestParam(value = "titlePic", required = false) String titlePic,
            @RequestParam(value = "create", required = false) String create,
            @RequestParam(value = "modified", required = false) String modified
    ) {
        TValueAddServiceType item = tValueAddServiceTypeService.get(id);
        if (null == item) {
            response.setStatus(404);
            return Resp.fail("the asset to be edited doesn't exist");
        }

        if( name != null ){
            item.setName(name);
        }
        if( type != null ){
            item.setType(type);
        }
        if( icon != null ){
            item.setIcon(icon);
        }
        if( iconsel != null ){
            item.setIconsel(iconsel);
        }
        if( titlePic != null ){
            item.setTitlePic(titlePic);
        }
        if( create != null ){
            item.setCreate(Tools.convertStringToDate(create, "yyyy-MM-dd HH:mm:ss"));
        }
        if( modified != null ){
            item.setModified(Tools.convertStringToDate(modified, "yyyy-MM-dd HH:mm:ss"));
        }
        tValueAddServiceTypeService.update(item);

        return tValueAddServiceTypeService.get(id);
    }

    @ApiOperation(value = "查询'TValueAddServiceType'表中的某条记录", notes = "根据url传入的数据id，查询对应的一条数据。")
    @RequestMapping(value = "/t_value_add_service_types/{t_value_add_service_type_id}",
                    method = {RequestMethod.GET, RequestMethod.DELETE, RequestMethod.PUT},
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTValueAddServiceType(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token") String token,
            @PathVariable(value = "t_value_add_service_type_id") Integer id,
            @RequestParam(value = "name", required = false) String name,
            @RequestParam(value = "type", required = false) Integer type,
            @RequestParam(value = "icon", required = false) String icon,
            @RequestParam(value = "iconsel", required = false) String iconsel,
            @RequestParam(value = "titlePic", required = false) String titlePic,
            @RequestParam(value = "create", required = false) String create,
            @RequestParam(value = "modified", required = false) String modified
    ) {
        if(request.getMethod().equals("GET")){
            TValueAddServiceType item = tValueAddServiceTypeService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            return item;
        }
        else if(request.getMethod().equals("PUT")){
            TValueAddServiceType item = tValueAddServiceTypeService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be edited doesn't exist");
            }

            if( name != null ){
                item.setName(name);
            }
            if( type != null ){
                item.setType(type);
            }
            if( icon != null ){
                item.setIcon(icon);
            }
            if( iconsel != null ){
                item.setIconsel(iconsel);
            }
            if( titlePic != null ){
                item.setTitlePic(titlePic);
            }
            if( create != null ){
                item.setCreate(Tools.convertStringToDate(create, "yyyy-MM-dd HH:mm:ss"));
            }
            if( modified != null ){
                item.setModified(Tools.convertStringToDate(modified, "yyyy-MM-dd HH:mm:ss"));
            }
            tValueAddServiceTypeService.update(item);

            return tValueAddServiceTypeService.get(id);
        }
        else if(request.getMethod().equals("DELETE")){
            TValueAddServiceType item = tValueAddServiceTypeService.get(id);
            if (null == item) {
                response.setStatus(404);
                return Resp.fail("the asset to be deleted doesn't exist");
            }

            tValueAddServiceTypeService.delete(id);
            return Resp.succ("delete success");
        }
        return Resp.fail("Unknown http method");
    }

    @ApiOperation(value = "查询'TValueAddServiceType'表中的多条记录或者新增某条记录", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。数据数量取决于pageNo和pageSize；数据的先后顺序取决于sortItem，sortOrder；   post请求：新增某条记录，传入表字段对应的值")
    @RequestMapping(value = "/t_value_add_service_types",
                    method = { RequestMethod.GET, RequestMethod.POST },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTValueAddServiceTypes(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "filters", required = false) String filters,
            @RequestParam(value = "includes", required = false) String includes,
            @RequestParam(value = "refers", required = false) String refers,
            @RequestParam(value = "relates", required = false) String relates
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortItem == null){
            sortItem = "id";
        }
        if(sortOrder == null){
            sortOrder = "asc";
        }
        if(filters == null){
            filters = "{}";
        }

        Long total = tValueAddServiceTypeService.countListWithPagingAndFilter(filters);
        response.addHeader("total", String.valueOf(total));

        return tValueAddServiceTypeService.getListWithPagingAndFilter(
                        pageNo, pageSize, sortItem, sortOrder, filters, includes, refers, relates);

    }

    @ApiOperation(value = "出入字段数组返回对应字段数据", notes = "get传参：根据url传入的filters（过滤条件），查询对应的多条数据。只返回你传进来的字段对应的数据")
    @RequestMapping(value = "/t_value_add_service_types/columns",
                    method = { RequestMethod.GET },
                    produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object getTValueAddServiceTypesId(
            HttpServletRequest request,
            @RequestHeader(value = "X-Auth-Token", required = false) String token,
            @RequestParam(value = "filters", required = true) String filters,
            @RequestParam(value = "columns", required = true) String columns,
            @RequestParam(value = "sortOrder", required = false) String sortOrder,
            @RequestParam(value = "sortItem", required = false) String sortItem,
            @RequestParam(value = "pageNo", required = false) Integer pageNo,
            @RequestParam(value = "pageSize", required = false) Integer pageSize
    ) {
        if(pageNo == null){
            pageNo = 1;
        }
        if(pageSize == null){
            pageSize = 10;
        }
        if(sortOrder==null){
            sortOrder = "asc";
        }
        if(sortItem==null){
            sortItem = "id";
        }
        JSONArray jsonArray = JSON.parseArray(columns);
        List<Map<String,Object>> lists  = new ArrayList<>();
        List<TValueAddServiceType> list = tValueAddServiceTypeService.getListWithPagingAndFilter(
                pageNo, pageSize, sortItem, sortOrder, filters);

        //反射调取get方法
        Class clz = TValueAddServiceType.class;
        list.stream().forEach(lot->{
            Map<String,Object> map = new HashMap<>();
            for (Object j : jsonArray){
                String s = (String) j;
                s = s.replaceFirst(s.substring(0,1),s.substring(0,1).toUpperCase());
                try {
                    if (clz.getMethod("get"+s).invoke(lot) !=null){
                        map.put((String) j,clz.getMethod("get"+s).invoke(lot));
                    }
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
            lists.add(map);
        });

        return lists;
     }



    @ApiOperation(value = "批量创建'TValueAddServiceType'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
    @RequestMapping(value = "/t_value_add_service_types/create//batch",
               method = RequestMethod.POST,
               produces = "application/json;charset=UTF-8")
    @ResponseBody
    public Object createsTValueAddServiceTypes(
             HttpServletRequest request,
             @RequestHeader(value = "X-Auth-Token", required = false) String token,
             @RequestParam(value = "params", required = false) String params
    ) {

             JSONArray jsonArray = JSON.parseArray(params);
             List<TValueAddServiceType> lists  = new ArrayList<>();

             jsonArray.stream().forEach((Object object) -> {

             JSONObject jsonObject = JSONObject.parseObject(object.toString());

             TValueAddServiceType item = new TValueAddServiceType();
             String id = Tools.getUUID();
                         item.setId(Integer.valueOf(id));
             if(jsonObject.containsKey("name")   ){
            item.setName(jsonObject.get("name").toString());
        }
        if(jsonObject.containsKey("type")   ){
            item.setType(Integer.valueOf(jsonObject.get("type").toString()));
        }
        if(jsonObject.containsKey("icon")   ){
            item.setIcon(jsonObject.get("icon").toString());
        }
        if(jsonObject.containsKey("iconsel")   ){
            item.setIconsel(jsonObject.get("iconsel").toString());
        }
        if(jsonObject.containsKey("titlePic")   ){
            item.setTitlePic(jsonObject.get("titlePic").toString());
        }
        if(jsonObject.containsKey("create")   ){
            item.setCreate(Tools.convertStringToDate(jsonObject.get("create").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
        if(jsonObject.containsKey("modified")   ){
            item.setModified(Tools.convertStringToDate(jsonObject.get("modified").toString(), "yyyy-MM-dd HH:mm:ss"));
        }
             tValueAddServiceTypeService.create(item);
             lists.add(tValueAddServiceTypeService.get(Integer.valueOf(id)));

             });
        return lists;
     }

    @ApiOperation(value = "批量删除'TValueAddServiceType'表中一条信息", notes = "通过post方法请求，传入表中字段的对应信息，创建一条数据。并返回给View层")
        @RequestMapping(value = "/t_value_add_service_types/delete/batch",
                method = RequestMethod.POST,
                produces = "application/json;charset=UTF-8")
        @ResponseBody
        public Object deletesTValueAddServiceType(
                HttpServletRequest request,
                @RequestHeader(value = "X-Auth-Token", required = false) String token,
                @RequestParam(value = "ids", required = true) String ids
        ) {

            JSONArray jsonArray = JSON.parseArray(ids);
            jsonArray.stream().forEach(id->{

                 TValueAddServiceType item = tValueAddServiceTypeService.get(Integer.valueOf(id.toString()));
                if (null != item) {
                    tValueAddServiceTypeService.delete(Integer.valueOf(id.toString()));
                }

            });
            return Resp.succ("delete success");
        }
}
